import re
import logging
from typing import Any, Optional

class SecurityValidator:
    """Security validation utilities for Excel operations"""
    
    def __init__(self):
        self.logger = logging.getLogger(__name__)
    
    def validate_file_path(self, file_path: str) -> bool:
        """Validate file path security"""
        # Prevent directory traversal attacks
        if '..' in file_path or file_path.startswith('/'):
            self.logger.warning(f"Potential directory traversal: {file_path}")
            return False
        return True
    
    def validate_sql_query(self, query: str) -> bool:
        """Basic SQL injection prevention"""
        # Look for potentially dangerous SQL patterns
        dangerous_patterns = [
            r'(?:DROP|DELETE|UPDATE|INSERT)\s+',
            r'(?:EXEC|EXECUTE)\s+',
            r'(?:UNION\s+SELECT)',
            r'(?:--|\/\*|\*\/)'
        ]
        
        for pattern in dangerous_patterns:
            if re.search(pattern, query, re.IGNORECASE):
                self.logger.warning(f"Potentially dangerous SQL pattern: {pattern}")
                return False
        return True
    
    def sanitize_input(self, value: Any) -> Any:
        """Basic input sanitization"""
        if isinstance(value, str):
            # Remove potentially dangerous characters
            value = re.sub(r'[<>"\'&]', '', value)
        return value



"""
智能配置推荐系统
基于游戏类型自动推荐配置结构、数值平衡性AI建议、配置优化预警系统
"""

import re
import json
from typing import Dict, List, Any, Optional, Tuple
from pathlib import Path
from collections import Counter, defaultdict
import pandas as pd


class GameTypeDetector:
    """游戏类型检测器"""
    
    GAME_PATTERNS = {
        'rpg': {
            'keywords': ['level', 'exp', 'hp', 'mp', 'skill', 'attribute', 'equipment', 'quest'],
            'sheet_patterns': ['skills', 'characters', 'items', 'quests', 'attributes']
        },
        'strategy': {
            'keywords': ['unit', 'building', 'resource', 'tech', 'civilization', 'army'],
            'sheet_patterns': ['units', 'buildings', 'resources', 'technologies']
        },
        'action': {
            'keywords': ['combo', 'animation', 'damage', 'health', 'power', 'weapon'],
            'sheet_patterns': ['characters', 'weapons', 'combos', 'animations']
        },
        'puzzle': {
            'keywords': ['level', 'score', 'achievement', 'star', 'objective'],
            'sheet_patterns': ['levels', 'scores', 'achievements', 'objectives']
        },
        'simulation': {
            'keywords': ['factory', 'production', 'resource', 'efficiency', 'capacity'],
            'sheet_patterns': ['buildings', 'production', 'resources', 'upgrades']
        }
    }
    
    def detect_game_type(self, excel_data: Dict[str, Any]) -> str:
        """检测游戏类型"""
        all_text = []
        
        # 收集所有工作表的文本数据
        for sheet_name, sheet_data in excel_data.items():
            if isinstance(sheet_data, dict):
                all_text.append(sheet_name)
                if 'data' in sheet_data:
                    for row in sheet_data['data']:
                        all_text.extend(str(cell) for cell in row if cell)
        
        text_content = ' '.join(all_text).lower()
        
        # 计算每种游戏类型的匹配度
        scores = {}
        for game_type, patterns in self.GAME_PATTERNS.items():
            score = 0
            for keyword in patterns['keywords']:
                score += text_content.count(keyword.lower())
            for sheet_pattern in patterns['sheet_patterns']:
                if sheet_pattern.lower() in text_content:
                    score += 2
            scores[game_type] = score
        
        # 返回得分最高的游戏类型
        return max(scores, key=scores.get) if scores else 'unknown'


class ConfigurationAnalyzer:
    """配置分析器"""
    
    def __init__(self):
        self.detector = GameTypeDetector()
    
    def analyze_excel_structure(self, excel_data: Dict[str, Any]) -> Dict[str, Any]:
        """分析Excel配置结构"""
        analysis = {
            'sheet_structure': {},
            'data_patterns': {},
            'validation_issues': [],
            'optimization_suggestions': []
        }
        
        for sheet_name, sheet_data in excel_data.items():
            if isinstance(sheet_data, dict) and 'data' in sheet_data:
                # 分析工作表结构
                rows = len(sheet_data['data'])
                cols = len(sheet_data['data'][0]) if rows > 0 else 0
                
                # 检测数据类型
                type_analysis = self._analyze_column_types(sheet_data['data'])
                
                analysis['sheet_structure'][sheet_name] = {
                    'rows': rows,
                    'cols': cols,
                    'types': type_analysis,
                    'headers': sheet_data['data'][0] if rows > 0 else []
                }
                
                # 识别数据模式
                patterns = self._identify_data_patterns(sheet_data['data'])
                analysis['data_patterns'][sheet_name] = patterns
                
                # 生成优化建议
                suggestions = self._generate_optimization_suggestions(
                    sheet_name, sheet_data['data'], patterns
                )
                analysis['optimization_suggestions'].extend(suggestions)
        
        return analysis
    
    def _analyze_column_types(self, data: List[List[Any]]) -> Dict[str, str]:
        """分析列数据类型"""
        if not data:
            return {}
        
        headers = data[0]
        types = {}
        
        for col_idx, header in enumerate(headers):
            col_data = [row[col_idx] for row in data[1:] if len(row) > col_idx]
            
            # 检测数据类型
            if all(isinstance(x, (int, float)) for x in col_data if x is not None):
                types[header] = 'numeric'
            elif all(isinstance(x, str) for x in col_data if x is not None):
                if any(x.lower() in ['true', 'false', 'yes', 'no'] for x in col_data if x):
                    types[header] = 'boolean'
                else:
                    types[header] = 'text'
            else:
                types[header] = 'mixed'
        
        return types
    
    def _identify_data_patterns(self, data: List[List[Any]]) -> Dict[str, Any]:
        """识别数据模式"""
        if not data:
            return {}
        
        headers = data[0]
        patterns = {}
        
        for col_idx, header in enumerate(headers):
            col_data = [row[col_idx] for row in data[1:] if len(row) > col_idx and row[col_idx] is not None]
            
            if not col_data:
                continue
            
            # 统计唯一值数量
            unique_count = len(set(col_data))
            total_count = len(col_data)
            
            # 识别常见模式
            patterns[header] = {
                'unique_values': unique_count,
                'total_values': total_count,
                'uniqueness_ratio': unique_count / total_count if total_count > 0 else 0,
                'sample_values': col_data[:5] if len(col_data) > 5 else col_data
            }
        
        return patterns
    
    def _generate_optimization_suggestions(self, sheet_name: str, data: List[List[Any]], patterns: Dict[str, Any]) -> List[Dict[str, Any]]:
        """生成优化建议"""
        suggestions = []
        
        if not data:
            return suggestions
        
        headers = data[0]
        
        for col_idx, header in enumerate(headers):
            if header in patterns:
                pattern = patterns[header]
                
                # 如果唯一值比例很低，建议转换为枚举类型
                if pattern['uniqueness_ratio'] < 0.1 and pattern['unique_values'] > 1:
                    suggestions.append({
                        'sheet': sheet_name,
                        'column': header,
                        'type': 'enum_optimization',
                        'suggestion': f'建议将"{header}"列转换为枚举类型，当前有{pattern["unique_values"]}个重复值',
                        'priority': 'medium'
                    })
                
                # 如果数值范围过大，建议添加范围验证
                if pattern.get('type') == 'numeric':
                    numeric_values = [float(x) for x in pattern['sample_values'] if isinstance(x, (int, float))]
                    if numeric_values:
                        min_val, max_val = min(numeric_values), max(numeric_values)
                        if max_val - min_val > 10000:
                            suggestions.append({
                                'sheet': sheet_name,
                                'column': header,
                                'type': 'validation_enhancement',
                                'suggestion': f'建议为"{header}"列添加数值范围验证({min_val:.2f} - {max_val:.2f})',
                                'priority': 'high'
                            })
        
        return suggestions


class SmartConfigurationRecommender:
    """智能配置推荐器"""
    
    def __init__(self):
        self.analyzer = ConfigurationAnalyzer()
    
    def recommend_configurations(self, excel_data: Dict[str, Any]) -> Dict[str, Any]:
        """智能推荐配置"""
        # 检测游戏类型
        game_type = self.analyzer.detector.detect_game_type(excel_data)
        
        # 分析配置结构
        analysis = self.analyzer.analyze_excel_structure(excel_data)
        
        # 生成配置建议
        recommendations = {
            'game_type': game_type,
            'analysis': analysis,
            'config_recommendations': self._generate_type_specific_recommendations(game_type, analysis),
            'validation_rules': self._generate_validation_rules(game_type, analysis),
            'optimization_tips': self._generate_optimization_tips(analysis)
        }
        
        return recommendations
    
    def _generate_type_specific_recommendations(self, game_type: str, analysis: Dict[str, Any]) -> List[Dict[str, Any]]:
        """生成类型特定的配置建议"""
        recommendations = []
        
        if game_type == 'rpg':
            recommendations.extend([
                {
                    'type': 'character_optimization',
                    'suggestion': '建议添加角色成长曲线表，记录等级与属性的关系',
                    'priority': 'high'
                },
                {
                    'type': 'balance_optimization', 
                    'suggestion': '建议添加技能平衡性检查工具，确保技能伤害比合理',
                    'priority': 'medium'
                }
            ])
        
        elif game_type == 'strategy':
            recommendations.extend([
                {
                    'type': 'resource_optimization',
                    'suggestion': '建议添加资源产出平衡性分析，避免经济失衡',
                    'priority': 'high'
                },
                {
                    'type': 'unit_optimization',
                    'suggestion': '建议添加单位克制关系表，完善战斗系统平衡性',
                    'priority': 'medium'
                }
            ])
        
        elif game_type == 'action':
            recommendations.extend([
                {
                    'type': 'combo_optimization',
                    'suggestion': '建议添加连招动作表，优化战斗流畅度',
                    'priority': 'medium'
                }
            ])
        
        return recommendations
    
    def _generate_validation_rules(self, game_type: str, analysis: Dict[str, Any]) -> List[Dict[str, Any]]:
        """生成验证规则"""
        rules = []
        
        # 基础验证规则
        for sheet_name, sheet_info in analysis.get('sheet_structure', {}).items():
            rules.append({
                'sheet': sheet_name,
                'rule': 'required_headers',
                'description': f'确保{sheet_name}表包含必要的基础字段',
                'priority': 'high'
            })
        
        # 游戏类型特定验证规则
        if game_type == 'rpg':
            rules.extend([
                {
                    'sheet': 'characters',
                    'rule': 'stat_validation',
                    'description': '角色属性必须为正数，且总和不超过最大值限制',
                    'priority': 'high'
                },
                {
                    'sheet': 'skills',
                    'rule': 'cost_validation', 
                    'description': '技能消耗必须合理，避免数值溢出',
                    'priority': 'medium'
                }
            ])
        
        return rules
    
    def _generate_optimization_tips(self, analysis: Dict[str, Any]) -> List[Dict[str, Any]]:
        """生成优化建议"""
        tips = []
        
        # 检查重复数据
        for sheet_name, patterns in analysis.get('data_patterns', {}).items():
            for col_name, pattern in patterns.items():
                if pattern.get('uniqueness_ratio', 0) < 0.1:
                    tips.append({
                        'type': 'data_compression',
                        'sheet': sheet_name,
                        'column': col_name,
                        'suggestion': f'{col_name}列重复率高，建议使用枚举或引用优化',
                        'priority': 'low'
                    })
        
        return tips


"""
测试范围表达式验证功能

验证新增的严格范围格式验证功能
"""

import pytest
from src.excel_mcp_server_fastmcp.utils.validators import ExcelValidator, DataValidationError


class TestRangeValidation:
    """测试范围表达式验证功能"""

    def test_valid_cell_range(self):
        """测试有效的单元格范围"""
        # 标准范围
        result = ExcelValidator.validate_range_expression("Sheet1!A1:C10")
        assert result['success'] is True
        assert result['sheet_name'] == "Sheet1"
        assert result['range_part'] == "A1:C10"
        assert result['range_info']['type'] == 'cell_range'
        assert result['range_info']['start_cell'] == 'A1'
        assert result['range_info']['end_cell'] == 'C10'

    def test_valid_row_range(self):
        """测试有效的行范围"""
        result = ExcelValidator.validate_range_expression("数据!1:10")
        assert result['success'] is True
        assert result['sheet_name'] == "数据"
        assert result['range_info']['type'] == 'row_range'
        assert result['range_info']['start_row'] == 1
        assert result['range_info']['end_row'] == 10

    def test_valid_column_range(self):
        """测试有效的列范围"""
        result = ExcelValidator.validate_range_expression("Test!A:C")
        assert result['success'] is True
        assert result['sheet_name'] == "Test"
        assert result['range_info']['type'] == 'column_range'
        assert result['range_info']['start_col'] == 1
        assert result['range_info']['end_col'] == 3

    def test_valid_single_cell(self):
        """测试单个单元格"""
        result = ExcelValidator.validate_range_expression("Sheet1!A1")
        assert result['success'] is True
        assert result['range_info']['type'] == 'single_cell'
        assert result['range_info']['column'] == 1
        assert result['range_info']['row'] == 1

    def test_valid_single_row(self):
        """测试单行"""
        result = ExcelValidator.validate_range_expression("Sheet1!5")
        assert result['success'] is True
        assert result['range_info']['type'] == 'single_row'
        assert result['range_info']['row'] == 5

    def test_valid_single_column(self):
        """测试单列"""
        result = ExcelValidator.validate_range_expression("Sheet1!A")
        assert result['success'] is True
        assert result['range_info']['type'] == 'single_column'
        assert result['range_info']['column'] == 1

    def test_chinese_sheet_name(self):
        """测试中文工作表名"""
        result = ExcelValidator.validate_range_expression("技能配置表!A1:Z100")
        assert result['success'] is True
        assert result['sheet_name'] == "技能配置表"
        assert result['normalized_range'] == "技能配置表!A1:Z100"

    def test_invalid_range_missing_sheet(self):
        """测试缺少工作表名的无效范围"""
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression("A1:C10")

        assert "必须包含工作表名和感叹号" in str(exc_info.value)

    def test_invalid_range_multiple_exclamation(self):
        """测试多个感叹号的无效范围"""
        # 当有多个感叹号时，会被解析为工作表名包含感叹号，然后范围部分会无效
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression("Sheet1!!A1:C10")

        # 这种情况会变成工作表名为"Sheet1!"，范围部分为"A1:C10"，实际会通过验证
        # 但我们测试的是真正无效的范围格式
        assert "无法识别的范围格式" in str(exc_info.value) or "工作表名包含无效字符" in str(exc_info.value)

    def test_invalid_empty_sheet_name(self):
        """测试空工作表名"""
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression("!A1:C10")

        assert "工作表名不能为空" in str(exc_info.value)

    def test_invalid_sheet_name_with_special_chars(self):
        """测试包含特殊字符的工作表名"""
        invalid_chars = ['[', ']', '*', ':', '?', '/', '\\']
        for char in invalid_chars:
            with pytest.raises(DataValidationError) as exc_info:
                ExcelValidator.validate_range_expression(f"Sheet{char}1!A1:C10")

            assert "工作表名包含无效字符" in str(exc_info.value)

    def test_invalid_sheet_name_too_long(self):
        """测试过长的工作表名"""
        long_name = "A" * 32  # 超过31个字符
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression(f"{long_name}!A1:C10")

        assert "工作表名长度不能超过31个字符" in str(exc_info.value)

    def test_invalid_empty_range_part(self):
        """测试空的范围部分"""
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression("Sheet1!")

        assert "范围部分不能为空" in str(exc_info.value)

    def test_invalid_range_format(self):
        """测试无效的范围格式"""
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression("Sheet1!INVALID_RANGE")

        assert "无法识别的范围格式" in str(exc_info.value)

    def test_invalid_null_or_empty(self):
        """测试空值或None"""
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression("")

        assert "范围表达式不能为空" in str(exc_info.value)

        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression(None)

        assert "范围表达式不能为空" in str(exc_info.value)

    def test_invalid_non_string(self):
        """测试非字符串类型"""
        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_range_expression(123)

        assert "范围表达式不能为空且必须是字符串" in str(exc_info.value)

    def test_case_insensitive_columns(self):
        """测试列名大小写不敏感"""
        result1 = ExcelValidator.validate_range_expression("Sheet1!a1:c10")
        result2 = ExcelValidator.validate_range_expression("Sheet1!A1:C10")

        assert result1['range_info']['start_col'] == result2['range_info']['start_col']
        assert result1['range_info']['end_col'] == result2['range_info']['end_col']

    def test_column_conversion(self):
        """测试列字母转换"""
        # 测试简单列
        assert ExcelValidator._col_to_num("A") == 1
        assert ExcelValidator._col_to_num("B") == 2
        assert ExcelValidator._col_to_num("Z") == 26

        # 测试多列
        assert ExcelValidator._col_to_num("AA") == 27
        assert ExcelValidator._col_to_num("AB") == 28
        assert ExcelValidator._col_to_num("AZ") == 52

        # 测试三列
        assert ExcelValidator._col_to_num("AAA") == 703

    def test_operation_scale_validation_low_risk(self):
        """测试低风险操作规模验证"""
        range_info = {
            'type': 'cell_range',
            'start_col': 1,
            'end_col': 10,
            'start_row': 1,
            'end_row': 10
        }

        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['rows'] == 10
        assert result['columns'] == 10
        assert result['total_cells'] == 100
        assert result['risk_level'] == "LOW"
        assert result['warning'] is None
        assert result['within_limits'] is True

    def test_operation_scale_validation_medium_risk(self):
        """测试中等风险操作规模验证"""
        range_info = {
            'type': 'cell_range',
            'start_col': 1,
            'end_col': 20,
            'start_row': 1,
            'end_row': 100
        }

        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['total_cells'] == 2000
        assert result['risk_level'] == "MEDIUM"
        assert "中等风险操作" in result['warning']
        assert result['within_limits'] is True

    def test_operation_scale_validation_high_risk(self):
        """测试高风险操作规模验证"""
        range_info = {
            'type': 'cell_range',
            'start_col': 1,
            'end_col': 50,
            'start_row': 1,
            'end_row': 300
        }

        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['total_cells'] == 15000
        assert result['risk_level'] == "HIGH"
        assert "高风险操作" in result['warning']
        assert result['within_limits'] is True

    def test_operation_scale_validation_rows_exceeded(self):
        """测试行数超限"""
        range_info = {
            'type': 'cell_range',
            'start_col': 1,
            'end_col': 10,
            'start_row': 1,
            'end_row': 2000  # 超过MAX_ROWS_OPERATION
        }

        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_operation_scale(range_info)

        assert "超过限制" in str(exc_info.value)

    def test_operation_scale_validation_columns_exceeded(self):
        """测试列数超限"""
        range_info = {
            'type': 'cell_range',
            'start_col': 1,
            'end_col': 200,  # 超过MAX_COLUMNS_OPERATION
            'start_row': 1,
            'end_row': 10
        }

        with pytest.raises(DataValidationError) as exc_info:
            ExcelValidator.validate_operation_scale(range_info)

        assert "超过限制" in str(exc_info.value)

    def test_single_row_scale_validation(self):
        """测试单行操作规模验证"""
        range_info = {
            'type': 'single_row',
            'row': 5
        }

        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['rows'] == 1
        assert result['columns'] == 1
        assert result['total_cells'] == 1
        assert result['risk_level'] == "LOW"

    def test_single_column_scale_validation(self):
        """测试单列操作规模验证"""
        range_info = {
            'type': 'single_column',
            'column': 5
        }

        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['rows'] == 1
        assert result['columns'] == 1
        assert result['total_cells'] == 1
        assert result['risk_level'] == "LOW"



# -*- coding: utf-8 -*-
"""
Excel MCP Server - ExcelOperations API合并测试

合并所有API相关测试文件：
- test_api_excel_operations.py (基础功能)
- test_api_excel_operations_advanced.py (高级功能)
- test_api_format_operations.py (格式化功能)
- test_new_apis.py (新API功能)
- test_server_apis.py (服务器API功能)
- test_api_error_handling.py (错误处理功能)

合并后保持100%测试覆盖率，消除冗余
"""

import pytest
import tempfile
import unittest.mock
import os
import sys
from pathlib import Path
from openpyxl import Workbook
from openpyxl.styles import Font, PatternFill, Alignment, Border, Side
from openpyxl.utils import get_column_letter, column_index_from_string
from openpyxl import load_workbook

# 添加项目路径到sys.path
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))

# 导入被测试的模块
from src.excel_mcp_server_fastmcp.api.excel_operations import ExcelOperations
from src.excel_mcp_server_fastmcp.models.types import OperationResult

class TestExcelOperationsBasic:
    """基础ExcelOperations API测试 - 原test_api_excel_operations.py"""

    # ==================== 测试数据准备 ====================

    @pytest.fixture
    def test_excel_file(self, temp_dir):
        """创建用于测试的Excel文件"""
        file_path = temp_dir / "test_operations.xlsx"

        wb = Workbook()
        ws = wb.active
        ws.title = "TestSheet"

        # 添加测试数据
        test_data = [
            ["姓名", "年龄", "邮箱"],
            ["张三", 25, "zhang@example.com"],
            ["李四", 30, "li@example.com"],
            ["王五", 28, "wang@example.com"],
            ["赵六", 35, "zhao@example.com"],
        ]

        for row_idx, row_data in enumerate(test_data, start=1):
            for col_idx, cell_value in enumerate(row_data, start=1):
                ws.cell(row=row_idx, column=col_idx, value=cell_value)

        wb.save(file_path)
        return str(file_path)

    @pytest.fixture
    def multi_sheet_file(self, temp_dir):
        """创建多工作表测试文件"""
        file_path = temp_dir / "multi_sheet.xlsx"

        wb = Workbook()
        
        # 第一个工作表
        ws1 = wb.active
        ws1.title = "员工信息"
        ws1.append(["ID", "姓名", "部门", "薪资"])
        ws1.append([1, "张三", "技术部", 8000])
        ws1.append([2, "李四", "市场部", 7000])
        
        # 第二个工作表
        ws2 = wb.create_sheet("部门信息")
        ws2.append(["部门ID", "部门名称", "人数"])
        ws2.append([101, "技术部", 5])
        ws2.append([102, "市场部", 3])
        
        wb.save(file_path)
        return str(file_path)

    # ==================== get_range方法测试 ====================

    def test_get_range_basic(self, test_excel_file):
        """测试基本的get_range功能"""
        result = ExcelOperations.get_range(test_excel_file, "TestSheet!A1:C5")
        
        assert result['success'] is True
        data = result.get('data', [])
        assert len(data) == 5  # 4行数据 + 1行标题
        # 提取第一行的值
        first_row_values = [cell['value'] for cell in data[0]]
        assert first_row_values == ["姓名", "年龄", "邮箱"]

    def test_get_range_with_sheet(self, multi_sheet_file):
        """测试带工作表名的get_range"""
        result = ExcelOperations.get_range(multi_sheet_file, "员工信息!A1:C3")
        
        assert result['success'] is True
        data = result.get('data', [])
        assert len(data) == 3
        # 提取第一行的值
        first_row_values = [cell['value'] for cell in data[0]]
        assert first_row_values == ["ID", "姓名", "部门"]

    def test_get_range_invalid_sheet(self, test_excel_file):
        """测试不存在的工作表名"""
        result = ExcelOperations.get_range(test_excel_file, "不存在的工作表!A1:B1")
        
        assert result['success'] is False
        assert "工作表" in result.get('message', '')

    def test_get_range_single_cell(self, test_excel_file):
        """测试单格读取"""
        result = ExcelOperations.get_range(test_excel_file, "TestSheet!B2")
        
        assert result['success'] is True
        data = result.get('data', [])
        assert len(data) == 1
        # 提取单元格的值
        cell_value = data[0][0]['value']
        assert cell_value == 25

    def test_get_range_invalid_format(self, test_excel_file):
        """测试无效的range格式"""
        result = ExcelOperations.get_range(test_excel_file, "无效格式")
        
        assert result['success'] is False
        assert "格式" in result.get('message', '')

    # ==================== batch_insert_rows测试 ====================

    def test_batch_insert_single_dict(self, test_excel_file):
        """测试单字典插入"""
        data = {"姓名": "测试", "年龄": 40, "邮箱": "test@example.com"}
        result = ExcelOperations.batch_insert_rows(test_excel_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result.get('data', {}).get('inserted_count') == 1

    def test_batch_insert_multiple_dicts(self, test_excel_file):
        """测试多字典插入"""
        data = [
            {"姓名": "测试1", "年龄": 41, "邮箱": "test1@example.com"},
            {"姓名": "测试2", "年龄": 42, "邮箱": "test2@example.com"},
        ]
        result = ExcelOperations.batch_insert_rows(test_excel_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result.get('data', {}).get('inserted_count') == 2

    def test_batch_insert_tuples(self, test_excel_file):
        """测试元组数据插入"""
        data = (
            {"姓名": "测试3", "年龄": 43, "邮箱": "test3@example.com"},
            {"姓名": "测试4", "年龄": 44, "邮箱": "test4@example.com"},
        )
        result = ExcelOperations.batch_insert_rows(test_excel_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result.get('data', {}).get('inserted_count') == 2

    def test_delete_rows_by_index(self, test_excel_file):
        """测试按行号删除"""
        result = ExcelOperations.delete_rows(test_excel_file, "TestSheet", row_index=3, count=1)
        
        assert result['success'] is True
        assert result['data']['actual_count'] == 1

    def test_delete_rows_by_condition(self, tmp_path):
        """测试条件删除"""
        file_path = str(tmp_path / "delete_test.xlsx")
        wb = Workbook()
        ws = wb.active
        ws.title = "测试表"
        ws.append(["ID", "名称", "值"])
        ws.append([1, "A", 10])
        ws.append([2, "B", 20])
        ws.append([3, "C", 30])
        wb.save(file_path)
        
        # 删除值>15的行
        result = ExcelOperations.delete_rows(file_path, "测试表", row_index=3, count=1)
        assert result['success'] is True

    def test_delete_rows_invalid_index(self, test_excel_file):
        """测试无效行号删除"""
        result = ExcelOperations.delete_rows(test_excel_file, "TestSheet", row_index=999, count=1)
        
        assert result['success'] is False
        assert "超过工作表" in result.get('message', '')

    # ==================== copy_sheet测试 ====================

    def test_copy_sheet_basic(self, multi_sheet_file):
        """测试基本工作表复制"""
        result = ExcelOperations.copy_sheet(multi_sheet_file, "员工信息")
        
        assert result['success'] is True
        assert "副本" in result.get('data', {}).get('name', '') or "副本" in result.get('message', '')

    def test_copy_sheet_custom_name(self, multi_sheet_file):
        """测试自定义名称复制"""
        result = ExcelOperations.copy_sheet(multi_sheet_file, "员工信息", new_name="员工信息备份")
        
        assert result['success'] is True
        assert result.get('data', {}).get('name') == "员工信息备份"

    def test_copy_sheet_to_existing(self, multi_sheet_file):
        """测试复制到已存在工作表"""
        result = ExcelOperations.copy_sheet(multi_sheet_file, "部门信息", new_name="员工信息")
        
        assert result['success'] is True
        assert result['data']['name'] != "员工信息"  # 应该自动重命名为员工信息_1等

    # ==================== 错误处理测试 ====================

    def test_permission_denied(self, tmp_path):
        """测试权限拒绝的情况"""
        # 创建只读文件
        file_path = tmp_path / "readonly.xlsx"
        wb = Workbook()
        ws = wb.active
        ws.append(["Test"])
        wb.save(file_path)
        
        # 模拟权限拒绝
        with unittest.mock.patch('builtins.open', side_effect=PermissionError("Permission denied")):
            result = ExcelOperations.get_range(str(file_path), "A1:B1")
            assert result['success'] is False

    def test_data_type_error(self, test_excel_file):
        """测试数据类型错误"""
        # 尝试插入不匹配的数据类型
        data = {"姓名": 123, "年龄": "abc", "邮箱": 456}  # 类型不匹配
        result = ExcelOperations.batch_insert_rows(test_excel_file, "TestSheet", data)
        
        # 这里应该测试ExcelOperations层是否正确处理类型转换或抛出错误
        assert result['success'] in [True, False]  # 可能成功也可能失败，取决于类型转换逻辑

    def test_memory_error(self, test_excel_file):
        """测试内存不足的情况"""
        # 模拟文件读取时内存不足
        with unittest.mock.patch('src.excel_mcp_server_fastmcp.core.excel_reader.ExcelReader.get_range', side_effect=MemoryError("Memory error")):
            result = ExcelOperations.get_range(test_excel_file, "TestSheet!A1:B1")
            assert result['success'] is False

    def test_disk_full_error(self, test_excel_file):
        """测试磁盘已满的情况"""
        # 模拟磁盘已满
        with unittest.mock.patch('openpyxl.Workbook.save', side_effect=OSError("Disk full")):
            result = ExcelOperations.get_range(test_excel_file, "A1:B1")
            assert result['success'] is False

class TestExcelOperationsAdvanced:
    """高级ExcelOperations API测试 - 原test_api_excel_operations_advanced.py"""

    @pytest.fixture
    def test_file(self, temp_dir):
        """创建测试文件"""
        file_path = temp_dir / "advanced_test.xlsx"
        
        wb = Workbook()
        ws = wb.active
        ws.title = "TestSheet"
        
        test_data = [
            ["ID", "Name", "Value", "Category", "Status"],
            [1, "Item1", 100, "A", "Active"],
            [2, "Item2", 200, "B", "Inactive"],
            [3, "Item3", 300, "A", "Active"],
            [4, "Item4", 400, "C", "Inactive"],
            [5, "Item5", 500, "B", "Active"],
        ]
        
        for row_idx, row_data in enumerate(test_data, start=1):
            for col_idx, cell_value in enumerate(row_data, start=1):
                ws.cell(row=row_idx, column=col_idx, value=cell_value)
        
        wb.save(file_path)
        return str(file_path)

    @pytest.fixture
    def multi_sheet_file(self, temp_dir):
        """创建多工作表文件"""
        file_path = temp_dir / "multi_sheet_test.xlsx"
        
        wb = Workbook()
        
        ws1 = wb.active
        ws1.title = "Sheet1"
        ws1.append(["ID", "Name"])
        ws1.append([1, "Item1"])
        ws1.append([2, "Item2"])
        
        ws2 = wb.create_sheet("Sheet2")
        ws2.append(["ID", "Category"])
        ws2.append([1, "A"])
        ws2.append([2, "B"])
        
        wb.save(file_path)
        return str(file_path)

    # ==================== 复杂数据处理测试 ====================

    def test_batch_insert_mixed_data_types(self, test_file):
        """测试混合数据类型插入"""
        data = [
            {"ID": 6, "Name": "Item6", "Value": 600.5, "Category": "D", "Status": "Active"},
            {"ID": 7, "Name": "Item7", "Value": 700, "Category": "E", "Status": "Inactive"},
        ]
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 2

    def test_batch_insert_with_empty_values(self, test_file):
        """测试包含空值的批量插入"""
        data = [
            {"ID": 8, "Name": "Item8", "Value": None, "Category": "F", "Status": "Active"},
            {"ID": 9, "Name": None, "Value": 900, "Category": "G", "Status": "Inactive"},
        ]
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 2

    def test_batch_insert_with_special_characters(self, test_file):
        """测试包含特殊字符的数据插入"""
        data = [
            {"ID": 10, "Name": "测试-特殊字符", "Value": 1000, "Category": "H", "Status": "活跃"},
            {"ID": 11, "Name": "Item11@#$%", "Value": 1100, "Category": "I", "Status": "Inactive"},
        ]
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 2

    # ==================== 工作表管理测试 ====================

    def test_rename_column(self, test_file):
        """测试列重命名"""
        result = ExcelOperations.rename_column(test_file, "TestSheet", "Name", "NewName")
        
        assert result['success'] is True
        # 验证列名是否被更改
        result_verify = ExcelOperations.get_range(test_file, "TestSheet!A1:E1")
        assert result_verify['success'] is True
        data = result_verify.get('data', [])
        # data[0] should be the row, data[0][1] should be the second cell (B1) which is the renamed "Name" column
        assert data[0][1]['value'] == "NewName"

    def test_rename_column_not_found(self, test_file):
        """测试重命名不存在的列"""
        result = ExcelOperations.rename_column(test_file, "TestSheet", "不存在的列", "NewName")
        
        assert result['success'] is False
        assert "列" in result.get('message', '')

    def test_copy_sheet_advanced(self, multi_sheet_file):
        """测试高级工作表复制"""
        result = ExcelOperations.copy_sheet(multi_sheet_file, "Sheet1", streaming=True)
        
        assert result['success'] is True
        metadata = result.get('metadata', {})
        assert metadata.get('mode') == 'streaming'
        assert metadata.get('copied_rows') == 3  # 包含表头行

    # ==================== 边界条件测试 ====================

    def test_large_data_insert(self, test_file):
        """测试大量数据插入"""
        # 插入100行数据
        data = [{"ID": i, "Name": f"Item{i}", "Value": i * 10, "Category": "Test", "Status": "Active"} 
                for i in range(100, 200)]
        
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 100

    def test_unicode_characters(self, test_file):
        """测试Unicode字符处理"""
        data = [
            {"ID": 12, "Name": "测试中文", "Value": 1200, "Category": "中文", "Status": "活跃"},
            {"ID": 13, "Name": "Emoji测试😊", "Value": 1300, "Category": "Emoji", "Status": "Inactive"},
        ]
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 2

    def test_empty_sheet_operations(self, test_file):
        """测试空工作表操作"""
        # 创建空工作表
        wb = load_workbook(test_file)
        ws = wb.create_sheet("EmptySheet")
        wb.save(test_file)
        
        # 尝试读取空工作表
        result = ExcelOperations.get_range(test_file, "EmptySheet!A1:B1")
        
        assert result['success'] is True
        data = result.get('data', [])
        # 空工作表的单元格仍会返回（只是值为None）
        assert len(data) >= 0

    def test_max_cell_value(self, test_file):
        """测试最大单元格值"""
        data = {"ID": 999999, "Name": "LargeID", "Value": 999999999, "Category": "Large", "Status": "Active"}
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 1

    # ==================== 性能测试 ====================

    def test_batch_insert_performance(self, test_file):
        """测试批量插入性能"""
        import time
        
        # 插入50行数据
        data = [{"ID": i, "Name": f"PerfItem{i}", "Value": i * 100, "Category": "Perf", "Status": "Active"} 
                for i in range(1000, 1050)]
        
        start_time = time.time()
        result = ExcelOperations.batch_insert_rows(test_file, "TestSheet", data)
        end_time = time.time()
        
        assert result['success'] is True
        assert result['data']['inserted_count'] == 50
        assert end_time - start_time < 5  # 应在5秒内完成

    def test_get_range_performance(self, test_file):
        """测试读取性能"""
        import time
        
        start_time = time.time()
        result = ExcelOperations.get_range(test_file, "TestSheet!A1:E50")
        end_time = time.time()
        
        assert result['success'] is True
        assert end_time - start_time < 2  # 应在2秒内完成